/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2010 Martin Rexa
 */

package cz.xelfi.quoridor.statistics.resources;

import com.sun.jersey.api.container.httpserver.HttpServerFactory;
import com.sun.jersey.api.core.PackagesResourceConfig;
import com.sun.jersey.api.core.ResourceConfig;
import com.sun.jersey.spi.resource.Singleton;
import com.sun.net.httpserver.HttpServer;
import java.io.File;
import java.io.IOException;
import java.util.Comparator;
import java.util.List;
import java.util.Collections;
import java.net.ServerSocket;
import javax.ws.rs.Path;
import cz.xelfi.quoridor.webidor.resources.Games;
import cz.xelfi.quoridor.webidor.Game;

/**
 *
 * @author Martin Rexa
 */
@Path("/")
@Singleton
public class Statistics {
    static final Comparator<Game> OLDEST_FIRST = new OldestFirst();
    static Elo elo;
    static Openings openings;

    @Path("elo")
    public Elo getElo() {
        return elo;
    }

    @Path("openings")
    public Openings getOpenings() {
        return openings;
    }

    public static void main( String[] args ) throws IOException, InterruptedException
    {
        int port = 9444;
        // timeout between reprocessing games in miliseconds
        long timeout = 1000 * 60 * 60 * 24;
//        long timeout = 1000 * 10;

        try {
            port = Integer.parseInt(args[0]);
        } catch (Exception ex) {
            // OK
        }

        if (System.getProperty("quoridor.dir") == null) {
            File home = new File(System.getProperty("user.home"));
            File quoridor = new File(home, ".quoridor");
            System.setProperty("quoridor.dir", quoridor.getPath());
        }

        long lastExec = System.currentTimeMillis();
        processGames();
        HttpServer s = start(port);
        System.out.println("Statistics started at port " + port);
        Object monitor = new Object();
        // Wait until begining of next period. If timeout is one day, then next
        // processing will be done after midnight
        synchronized (monitor) {
            monitor.wait(timeout - (lastExec % timeout));
        }
        for(;;){
            processGames();
            synchronized (monitor) {
                monitor.wait(timeout);
            }
        }
    }

    public static HttpServer start(int port) throws IOException {
        if (port == -1) {
            ServerSocket ss = new ServerSocket(0);
            port =ss.getLocalPort();
            ss.close();
        }
        final String baseUri = "http://localhost:" + port + "/";

        if (System.getProperty("quoridor.dir") == null) {
            File home = new File(System.getProperty("user.home"));
            File quoridor = new File(home, ".quoridor");
            System.setProperty("quoridor.dir", quoridor.getPath());
        }

        ResourceConfig rc = new PackagesResourceConfig("cz.xelfi.quoridor.statistics");
        HttpServer server = HttpServerFactory.create(baseUri, rc);
        server.start();
        return server;
    }

    public static void processGames(){
        Elo eloTmp = new Elo();
        Openings openingsTmp = new Openings();

        final String prop = System.getProperty("quoridor.dir"); // NOI18N
        if (prop == null) {
            throw new IllegalStateException("quoridor.dir property must be specified"); // NOI18N
        }
        File path = new File(prop);
        path.mkdirs();
        File fGames = new File(path, "games");
        //fGames.mkdirs();
        Games games = new Games(fGames, null);
        List<Game> lGames = games.getGames();
        Collections.sort(lGames, OLDEST_FIRST);

        // process games
        for(Game g: lGames){
            if(g.getId().isFinished()){
                eloTmp.processGame(g.getId());
                openingsTmp.processGame(g);
            }
        }

        elo = eloTmp;
        openings = openingsTmp;
    }

    private static final class OldestFirst implements Comparator<Game> {
        public int compare(Game g1, Game g2) {
            if(g1.getId() == g2.getId())
                return 0;
            long diff = g2.getId().getModified() - g1.getId().getModified();
            if (diff != 0) {
                return diff < 0 ? 1 : -1;
            }
            return g1.getId().getId().compareTo(g2.getId().getId());
        }
    }

}
